@@ -1,4 +1,4 @@
 # flake8: noqa: F401
 from .pymongo import pymongo
 from .connection import Connection
-from .base_object import BaseObject
+from .preload import preload
