@@ -72,28 +72,6 @@ class Encoder(Coder):
             commands.extend(item_commands)
         return EncodingResult(result, commands)
 
-    def encode_shape(self, context: EncodingContext) -> EncodingResult:
-        if context.value is None:
-            return EncodingResult(result=None, commands=[])
-        value = cast(dict[str, Any], context.value)
-        fd = context.types.fdef
-        shape_types = cast(dict[str, Any], fd.shape_types)
-        camelized = context.owner.__class__.pconf.camelize_db_keys
-        result = {}
-        commands = []
-        for key, item in value.items():
-            item_types = shape_types[key]
-            item_result, item_commands = self.encode_item(context.new(
-                value=item,
-                types=item_types,
-                keypath_root=concat_keypath(context.keypath_root, key),
-                keypath_owner=concat_keypath(context.keypath_owner, key),
-                keypath_parent=str(key),
-                parent=value))
-            result[camelize(key, False) if camelized else key] = item_result
-            commands.extend(item_commands)
-        return EncodingResult(result, commands)
-
     def _join_command(self,
                       this_instance: PymongoObject,
                       this_field: JField,
@@ -110,7 +88,7 @@ class Encoder(Coder):
             this_field.name,
             that_cls,
             cast(str, this_field.fdef.foreign_key))
-        collection = connection.collection(join_table_name)
+        collection = connection.collection(join_table_name, [this_field_name, that_field_name])
         this_id = ObjectId(this_instance._id)
         matcher = {
             this_field_name: this_id,
@@ -136,7 +114,7 @@ class Encoder(Coder):
             this_field.name,
             that_cls,
             cast(str, this_field.fdef.foreign_key))
-        collection = connection.collection(join_table_name)
+        collection = connection.collection(join_table_name, [this_field_name, that_field_name])
         this_id = ObjectId(this_instance._id)
         matcher = {
             this_field_name: this_id,
@@ -165,18 +143,23 @@ class Encoder(Coder):
         if value.is_new:
             use_insert_command = True
         result_set = {}
+        result_addtoset = {}
+        result_unset = {}
         matcher = {}
         commands = []
-        result_addtoset = {}
         for field in value.__class__.cdef.fields:
-            if field.fdef.is_temp_field:
+            if field.fdef.fstore == FStore.TEMP:
+                continue
+            if field.fdef.fstore == FStore.CALCULATED:
                 continue
             fname = field.name
             fvalue = getattr(value, fname)
             ftypes = field.types
             if self.is_id_field(field):
                 result_set['_id'] = ObjectId(fvalue)
-                if not use_insert_command:
+                if use_insert_command:
+                    pass
+                else:
                     matcher['_id'] = ObjectId(fvalue)
             elif self.is_foreign_key_reference_field(field):
                 if fvalue is None:
@@ -212,14 +195,32 @@ class Encoder(Coder):
                             self.list_instance_type(field),
                             list_item['_id'])
                         commands.append(join_command)
-                for item in value.unlinked_objects[field.name]:
-                    if item._id is not None:
+                if value.unlinked_objects.get(field.name) is not None:
+                    for item in value.unlinked_objects[field.name]:
+                        if item._id is not None:
+                            unlink_command = self._unlink_command(
+                                value,
+                                field,
+                                self.list_instance_type(field),
+                                ObjectId(item._id))
+                            commands.append(unlink_command)
+                if value._link_keys.get(field.name) is not None:
+                    for k in value._link_keys.get(field.name):
+                        join_command = self._join_command(
+                            value,
+                            field,
+                            self.list_instance_type(field),
+                            ObjectId(k))
+                        commands.append(join_command)
+                if value._unlink_keys.get(field.name) is not None:
+                    for k in value._unlink_keys.get(field.name):
                         unlink_command = self._unlink_command(
                             value,
                             field,
                             self.list_instance_type(field),
-                            ObjectId(item._id))
+                            ObjectId(k))
                         commands.append(unlink_command)
+
             elif self.is_local_key_reference_field(field):
                 if fvalue is None:
                     tsfm = value.__class__.cdef.jconf.ref_key_encoding_strategy
@@ -228,8 +229,10 @@ class Encoder(Coder):
                             result_set[ref_db_field_key(fname, cls)] = \
                                 ObjectId(getattr(value, tsfm(field)))
                     else:
-                        if use_insert_command or fname in fields_need_update:
-                            result_set[ref_db_field_key(fname, cls)] = None
+                        if use_insert_command:
+                            pass
+                        elif fname in fields_need_update:
+                            result_unset[ref_db_field_key(fname, cls)] = None
                     continue
                 item_result, item_commands = self.encode_instance(context.new(
                     value=fvalue,
@@ -276,10 +279,17 @@ class Encoder(Coder):
                     keypath_parent=fname,
                     parent=value))
                 if use_insert_command or fname in fields_need_update:
+                    efname = fname
                     if context.owner.__class__.pconf.camelize_db_keys:
-                        result_set[camelize(field.name, False)] = item_result
+                        efname = camelize(field.name, False)
+                    if use_insert_command:
+                        if item_result is not None:
+                            result_set[efname] = item_result
                     else:
-                        result_set[field.name] = item_result
+                        if item_result is None:
+                            result_unset[efname] = None
+                        else:
+                            result_set[efname] = item_result
                 commands.extend(item_commands)
         if write_instance:
             collection = Connection.get_collection(value.__class__)
@@ -292,6 +302,8 @@ class Encoder(Coder):
                     updator['$set'] = result_set
                 if len(result_addtoset) > 0:
                     updator['$addToSet'] = result_addtoset
+                if len(result_unset) > 0:
+                    updator['$unset'] = result_unset
                 if len(updator) > 0:
                     update_c = UpdateOneCommand(collection, updator, matcher)
                     commands.append(update_c)
@@ -310,8 +322,6 @@ class Encoder(Coder):
             return self.encode_list(context)
         elif ftype == FType.DICT:
             return self.encode_dict(context)
-        elif ftype == FType.SHAPE:
-            return self.encode_shape(context)
         elif ftype == FType.INSTANCE:
             return self.encode_instance(context)
         elif ftype == FType.DATE:
@@ -326,7 +336,7 @@ class Encoder(Coder):
     def encode_root(self, root: T) -> BatchCommand:
         commands = self.encode_instance(EncodingContext(
             value=root,
-            types=types.instanceof(root.__class__),
+            types=types.objof(root.__class__),
             keypath_root='',
             root=root,
             keypath_owner='',
