@@ -33,11 +33,11 @@ class PymongoObject(JObject):
         ...
 
     @classmethod
-    def id(cls: type[T], id: Union[str, ObjectId]) -> IDQuery[T]:
+    def id(cls: type[T], id: str | ObjectId, *args, **kwargs: Any) -> IDQuery[T]:
         ...
 
     @classmethod
-    def linked(cls: type[T]) -> BaseQuery[T]:
+    def linked(cls: type[T], *args, **kwargs: Any) -> BaseQuery[T]:
         ...
 
     @classmethod
