@@ -6,6 +6,7 @@ from jsonclasses.jfield import JField
 from pymongo.errors import DuplicateKeyError
 from inflection import camelize, underscore
 from pymongo.collection import Collection
+from pymongo import ASCENDING
 from jsonclasses.fdef import FStore, FType
 from jsonclasses.excs import UniqueConstraintException
 from jsonclasses.excs import DeletionDeniedException
@@ -13,7 +14,7 @@ from .pymongo_object import PymongoObject
 from .query import BaseQuery, ExistQuery, IterateQuery, ListQuery, SingleQuery, IDQuery
 from .encoder import Encoder
 from .connection import Connection
-from .utils import btype_from_ftype, ref_db_field_key
+from .utils import ref_db_field_key
 from .coder import Coder
 
 
@@ -34,12 +35,18 @@ def one(cls: type[T], *args, **kwargs: Any) -> SingleQuery[T]:
         return SingleQuery(cls=cls, filter=kwargs)
 
 
-def pymongo_id(cls: type[T], id: Union[str, ObjectId]) -> IDQuery[T]:
-    return IDQuery(cls=cls, id=id)
+def pymongo_id(cls: type[T], id: str | ObjectId, *args, **kwargs: Any) -> IDQuery[T]:
+    if len(args) > 0:
+        return IDQuery(cls=cls, id=id, matcher=args[0])
+    else:
+        return IDQuery(cls=cls, id=id, matcher=kwargs)
 
 
-def linked(cls: type[T]) -> BaseQuery[T]:
-    return BaseQuery(cls=cls)
+def linked(cls: type[T], *args, **kwargs: Any) -> BaseQuery[T]:
+    if len(args) > 0:
+        return SingleQuery(cls=cls, filter=args[0])
+    else:
+        return SingleQuery(cls=cls, filter=kwargs)
 
 
 def exist(cls: type[T], **kwargs: Any) -> ExistQuery[T]:
@@ -54,14 +61,28 @@ def _database_write(self: T) -> None:
     try:
         Encoder().encode_root(self).execute()
     except DuplicateKeyError as exception:
-        result = search('index: (.+?)_1', exception._message)
+        result = search('index: (.+?) dup key', exception._message)
         assert result is not None
-        db_key = result.group(1)
-        pt_key = db_key
-        if self.__class__.pconf.camelize_db_keys:
-            pt_key = underscore(db_key)
-        raise UniqueConstraintException(pt_key) from None
-
+        index_key = result.group(1)
+        single_key = True
+        if index_key.endswith('_1'):
+            db_key = index_key[:-2]
+            single_key = True
+        else:
+            db_key = index_key
+            single_key = False
+        if single_key:
+            pt_key = db_key
+            if self.__class__.pconf.camelize_db_keys:
+                pt_key = underscore(db_key)
+            raise UniqueConstraintException(pt_key) from None
+        else:
+            results = []
+            for field in self.__class__.cdef.fields:
+                if field.fdef.cindex and db_key in field.fdef.cindex_names:
+                    results.append(field.name)
+            ek = self.__class__.cdef.jconf.key_encoding_strategy
+            raise UniqueConstraintException([ek(r) for r in results], f'voilated unique compound index \'{index_key}\'')
 
 def _orm_delete(self: T, no_raise: bool = False) -> None:
     # deny test
@@ -167,6 +188,25 @@ def _orm_delete(self: T, no_raise: bool = False) -> None:
     setattr(self, '_is_deleted', True)
 
 
+def _orm_complete(self: T) -> None:
+    this_pick = []
+    mfields = self.modified_fields
+    for field in self.__class__.cdef.fields:
+        if field.fdef.fstore == FStore.EMBEDDED:
+            if field.name not in self._partial_picks:
+                this_pick.append(field.name)
+        elif field.fdef.fstore == FStore.LOCAL_KEY:
+            kr = self.__class__.cdef.jconf.ref_key_encoding_strategy
+            ref_key = kr(field)
+            if ref_key not in self._partial_picks:
+                this_pick.append(ref_key)
+    result = self.__class__.id(self._id, {'_pick': this_pick}).exec()
+    for k in this_pick:
+        if k not in mfields:
+            setattr(self, k, getattr(result, k))
+    setattr(self, '_is_partial', False)
+
+
 def _orm_restore(self: T) -> None:
     pass
 
@@ -188,40 +228,91 @@ def pymongofy(class_: type) -> PymongoObject:
     class_._database_write = _database_write
     class_._orm_delete = _orm_delete
     class_._orm_restore = _orm_restore
+    class_._orm_complete = _orm_complete
     connection = Connection.from_class(class_)
     if class_.cdef.jconf.abstract:
         return class_
     def callback(coll: Collection):
         info = coll.index_information()
+        existing_index_keys = list(info.keys())
+        compound_fields: dict[str, list[JField]] = {}
+        compound_ufields: dict[str, list[JField]] = {}
         for field in class_.cdef.fields:
-            name = field.name
+            fname = field.name
             if class_.pconf.camelize_db_keys:
-                name = camelize(field.name, False)
+                fname = camelize(field.name, False)
             index = field.fdef.index
             unique = field.fdef.unique
             required = field.fdef.required
-            index_name = f'{name}_1'
-            ftype = field.fdef.ftype
-            if unique:
-                if required:
-                    coll.create_index(name, name=index_name, unique=True)
-                else:
-                    coll.create_index(
-                        name, name=index_name, unique=True,
-                        partialFilterExpression={
-                            name: {'$type': btype_from_ftype(ftype)}
-                        })
-            elif index:
-                if required:
-                    coll.create_index(name, name=index_name)
-                else:
-                    coll.create_index(
-                        name, name=index_name,
-                        partialFilterExpression={
-                            name: {'$type': btype_from_ftype(ftype)}
-                        })
+            index_name = f'{fname}_1'
+            cindex = field.fdef.cindex
+            cindex_names = field.fdef.cindex_names
+            cunique = field.fdef.cunique
+            cunique_names = field.fdef.cunique_names
+            # check single index
+            if unique or index:
+                coll.create_index(fname, name=index_name, unique=unique, sparse=not required)
+                if index_name in existing_index_keys:
+                    existing_index_keys.remove(index_name)
             else:
-                if index_name in info.keys():
+                if index_name in existing_index_keys:
                     coll.drop_index(index_name)
+                    existing_index_keys.remove(index_name)
+            # check compound index
+            if cindex:
+                for ciname in cindex_names:
+                    if ciname not in compound_fields:
+                        compound_fields[ciname] = []
+                    if field.fdef.fstore == FStore.LOCAL_KEY:
+                        res = field.fdef.cdef.jconf.ref_key_encoding_strategy
+                        compound_fields[ciname].append(field)
+                    else:
+                        compound_fields[ciname].append(field)
+            if cunique:
+                for cuname in cunique_names:
+                    if cuname not in compound_ufields:
+                        compound_ufields[cuname] = []
+                    if field.fdef.fstore == FStore.LOCAL_KEY:
+                        res = field.fdef.cdef.jconf.ref_key_encoding_strategy
+                        compound_ufields[cuname].append(field)
+                    else:
+                        compound_ufields[cuname].append(field)
+        for index_name, fields in compound_fields.items():
+            sparse: bool = False
+            keys: list[tuple[str, Any]] = []
+            for field in fields:
+                if field.fdef.fstore == FStore.LOCAL_KEY:
+                    res = field.fdef.cdef.jconf.ref_key_encoding_strategy
+                    key = res(field)
+                else:
+                    key = field.name
+                if class_.pconf.camelize_db_keys:
+                    key = camelize(key, False)
+                if not field.fdef.required:
+                    sparse = True
+                keys.append((key, ASCENDING))
+            coll.create_index(keys, name=index_name, sparse=sparse)
+            if index_name in existing_index_keys:
+                existing_index_keys.remove(index_name)
+        for index_name, fields in compound_ufields.items():
+            sparse: bool = False
+            keys: list[tuple[str, Any]] = []
+            for field in fields:
+                if field.fdef.fstore == FStore.LOCAL_KEY:
+                    res = field.fdef.cdef.jconf.ref_key_encoding_strategy
+                    key = res(field)
+                else:
+                    key = field.name
+                if class_.pconf.camelize_db_keys:
+                    key = camelize(key, False)
+                if not field.fdef.required:
+                    sparse = True
+                keys.append((key, ASCENDING))
+            coll.create_index(keys, name=index_name, unique=True, sparse=sparse)
+            if index_name in existing_index_keys:
+                existing_index_keys.remove(index_name)
+        for left_key in existing_index_keys:
+            if left_key != '_id_':
+                coll.drop_index(left_key)
     connection.add_connected_callback(class_.pconf.collection_name, callback)
     return class_
