@@ -7,8 +7,11 @@ from tests.classes.linked_author import LinkedAuthor
 from tests.classes.linked_post import LinkedPost
 from tests.classes.linked_profile_user import LinkedProfile, LinkedUser
 from tests.classes.linked_favorite import LinkedCourse, LinkedStudent
-from tests.classes.chained_user import (ChainedAddress, ChainedProfile,
-                                        ChainedUser)
+from tests.classes.chained_user import (
+    ChainedAddress, ChainedProfile, ChainedUser
+)
+from tests.classes.linked_song import LinkedSong, LinkedSinger
+
 
 
 class TestSave(TestCase):
@@ -60,6 +63,10 @@ class TestSave(TestCase):
         collection.delete_many({})
         collection = Connection.get_collection(ChainedAddress)
         collection.delete_many({})
+        collection = Connection.get_collection(LinkedSong)
+        collection.delete_many({})
+        collection = Connection.get_collection(LinkedSinger)
+        collection.delete_many({})
 
     def test_1f_1l_ref_lookup_fetches_linked_object(self):
         user = LinkedUser(name='Teo Yeo')
@@ -70,6 +77,24 @@ class TestSave(TestCase):
         self.assertEqual(user.profile.id, profile.id)
         self.assertEqual(user.profile.name, profile.name)
 
+    def test_1f_1l_ref_lookup_fetches_linked_object_use_id_include(self):
+        user = LinkedUser(name='Teo Yeo')
+        profile = LinkedProfile(name='Great City')
+        user.profile = profile
+        user.save()
+        user = LinkedUser.id(user.id, **{'_includes': ['profile']}).exec()
+        self.assertEqual(user.profile.id, profile.id)
+        self.assertEqual(user.profile.name, profile.name)
+
+    def test_1f_1l_ref_lookup_fetches_linked_object_use_id_str_include(self):
+        user = LinkedUser(name='Teo Yeo')
+        profile = LinkedProfile(name='Great City')
+        user.profile = profile
+        user.save()
+        user = LinkedUser.id(user.id, '_includes[0]=profile').exec()
+        self.assertEqual(user.profile.id, profile.id)
+        self.assertEqual(user.profile.name, profile.name)
+
     def test_1l_1f_ref_lookup_fetches_linked_object(self):
         user = LinkedUser(name='Teo Yeo')
         profile = LinkedProfile(name='Great City')
@@ -325,3 +350,123 @@ class TestSave(TestCase):
         self.assertEqual(students[1].courses[1].is_new, False)
         self.assertEqual(students[1].courses[1].is_modified, False)
         self.assertEqual(students[1].courses[1].modified_fields, ())
+
+    def test_fetched_objects_are_sorted_many_many(self):
+        student1 = LinkedStudent(name='S1')
+        student2 = LinkedStudent(name='S2')
+        course1 = LinkedCourse(name='C1')
+        course2 = LinkedCourse(name='C2')
+        student1.courses = [course1, course2]
+        student2.courses = [course1, course2]
+        student1.save()
+        students = LinkedStudent.find().include('courses', LinkedCourse.find().order('name', -1)).exec()
+        self.assertEqual(students[0].courses[0].name, 'C2')
+        self.assertEqual(students[0].courses[1].name, 'C1')
+        self.assertEqual(students[1].courses[0].name, 'C2')
+        self.assertEqual(students[1].courses[1].name, 'C1')
+        students = LinkedStudent.find().include('courses', LinkedCourse.find().order('name', 1)).exec()
+        self.assertEqual(students[0].courses[0].name, 'C1')
+        self.assertEqual(students[0].courses[1].name, 'C2')
+        self.assertEqual(students[1].courses[0].name, 'C1')
+        self.assertEqual(students[1].courses[1].name, 'C2')
+
+    def test_fetched_objects_are_filtered_many_many(self):
+        student1 = LinkedStudent(name='S1')
+        student2 = LinkedStudent(name='S2')
+        course1 = LinkedCourse(name='C1')
+        course2 = LinkedCourse(name='C2')
+        student1.courses = [course1, course2]
+        student2.courses = [course1, course2]
+        student1.save()
+        students = LinkedStudent.find().include('courses', LinkedCourse.find(name='C1')).exec()
+        self.assertEqual(len(students[0].courses), 1)
+        self.assertEqual(len(students[1].courses), 1)
+
+    def test_fetched_objects_are_paged_many_many(self):
+        student = LinkedStudent(name='S1')
+        for i in range(100):
+            course = LinkedCourse(name=f's{str(i).zfill(3)}')
+            student.courses.append(course)
+        student.save()
+        student = LinkedStudent.one().include('courses', LinkedCourse.find().order('name', -1).page_size(10).page_number(2)).exec()
+        course_names = [course.name for course in student.courses]
+        self.assertEqual(course_names, ['s089', 's088', 's087', 's086', 's085', 's084', 's083', 's082', 's081', 's080'])
+        student = LinkedStudent.one().include('courses', LinkedCourse.find().order('name', -1).page_size(10).page_number(4)).exec()
+        course_names = [course.name for course in student.courses]
+        self.assertEqual(course_names, ['s069', 's068', 's067', 's066', 's065', 's064', 's063', 's062', 's061', 's060'])
+        student = LinkedStudent.one().include('courses', LinkedCourse.find().order('name', 1).page_size(8).page_number(1)).exec()
+        course_names = [course.name for course in student.courses]
+        self.assertEqual(course_names, ['s000', 's001', 's002', 's003', 's004', 's005', 's006', 's007'])
+
+    def test_fl_many_many_are_included_from_f_side(self):
+        singer1 = LinkedSinger(name='Teh Khim Leng')
+        singer2 = LinkedSinger(name='M Teh Khim Leng')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        song2 = LinkedSong(name='Teo Sua Si Gueh Hai', singers=[singer1, singer2]).save()
+        song3 = LinkedSong(name='Siao Thiang Go', singers=[singer1]).save()
+        result = LinkedSinger.id(singer2.id).include('songs').exec()
+        self.assertEqual(len(result.songs), 2)
+        self.assertEqual(result.songs[0].id, song1.id)
+        self.assertEqual(result.songs[1].id, song2.id)
+
+    def test_fl_many_many_are_included_from_l_side(self):
+        singer1 = LinkedSinger(name='Teh Khim Leng')
+        singer2 = LinkedSinger(name='M Teh Khim Leng')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        singer3 = LinkedSinger(name='Phua Kheng Lim')
+        singer4 = LinkedSinger(name='M Phua Kheng Lim')
+        LinkedSong(name='Teo Sua Nang To Cim Tsung Ci', singers=[singer3, singer4]).save()
+        result = LinkedSong.one(name='Lu Bo To Teo Yeo').include('singers').exec()
+        self.assertEqual(result.singer_ids, [singer1.id, singer2.id])
+        self.assertEqual(result.singers[0].id, singer1.id)
+        self.assertEqual(result.singers[1].id, singer2.id)
+
+    def test_fl_many_many_are_included_from_f_side_with_filter(self):
+        singer1 = LinkedSinger(name='Teh Khim Leng')
+        singer2 = LinkedSinger(name='M Teh Khim Leng')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        song2 = LinkedSong(name='Teo Sua Si Gueh Hai', singers=[singer1, singer2]).save()
+        song3 = LinkedSong(name='Siao Thiang Go', singers=[singer1]).save()
+        result = LinkedSinger.id(singer2.id).include('songs', LinkedSong.find(name={'_prefix': 'L'})).exec()
+        self.assertEqual(len(result.songs), 1)
+        self.assertEqual(result.songs[0].id, song1.id)
+
+    def test_fl_many_many_are_included_from_l_side_with_filter(self):
+        singer1 = LinkedSinger(name='Teh Khim Leng')
+        singer2 = LinkedSinger(name='M Teh Khim Leng')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        singer3 = LinkedSinger(name='Phua Kheng Lim')
+        singer4 = LinkedSinger(name='M Phua Kheng Lim')
+        LinkedSong(name='Teo Sua Nang To Cim Tsung Ci', singers=[singer3, singer4]).save()
+        result = LinkedSong.one(name='Lu Bo To Teo Yeo').include('singers', LinkedSinger.find(name={'_prefix': 'M'})).exec()
+        self.assertEqual(len(result.singers), 1)
+        self.assertEqual(result.singers[0].id, singer2.id)
+
+    def test_fl_many_many_are_included_from_f_side_with_sort(self):
+        singer1 = LinkedSinger(name='Teh Khim Leng')
+        singer2 = LinkedSinger(name='M Teh Khim Leng')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        song2 = LinkedSong(name='Lw Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        song3 = LinkedSong(name='Li Bo To Teo Yeo', singers=[singer1, singer2]).save()
+        song4 = LinkedSong(name='Teo Sua Si Gueh Hai', singers=[singer1, singer2]).save()
+        song5 = LinkedSong(name='Siao Thiang Go', singers=[singer1]).save()
+        result = LinkedSinger.id(singer2.id).include('songs', LinkedSong.find(name={'_prefix': 'L'}).order('name', -1)).exec()
+        self.assertEqual(len(result.songs), 3)
+        self.assertEqual([song.name for song in result.songs], ['Lw Bo To Teo Yeo', 'Lu Bo To Teo Yeo', 'Li Bo To Teo Yeo'])
+        result = LinkedSinger.id(singer2.id).include('songs', LinkedSong.find(name={'_prefix': 'L'}).order('name', 1)).exec()
+        self.assertEqual(len(result.songs), 3)
+        self.assertEqual([song.name for song in result.songs], ['Li Bo To Teo Yeo', 'Lu Bo To Teo Yeo', 'Lw Bo To Teo Yeo'])
+
+    def test_fl_many_many_are_included_from_l_side_with_sort_and_sort_will_not_work(self):
+        singer1 = LinkedSinger(name='AQ')
+        singer2 = LinkedSinger(name='BQ')
+        singer3 = LinkedSinger(name='CC')
+        singer4 = LinkedSinger(name='DD')
+        song1 = LinkedSong(name='Lu Bo To Teo Yeo', singers=[singer2, singer1, singer3, singer4]).save()
+        LinkedSong(name='Teo Sua Nang To Cim Tsung Ci', singers=[singer3, singer4]).save()
+        result = LinkedSong.one(name='Lu Bo To Teo Yeo').include('singers', LinkedSinger.find(name={'_suffix': 'Q'}).order('name', -1)).exec()
+        self.assertEqual(len(result.singers), 2)
+        self.assertEqual([singer.name for singer in result.singers], ['BQ', 'AQ'])
+        result = LinkedSong.one(name='Lu Bo To Teo Yeo').include('singers', LinkedSinger.find(name={'_suffix': 'Q'}).order('name', 1)).exec()
+        self.assertEqual([singer.name for singer in result.singers], ['BQ', 'AQ'])
+        self.assertEqual(len(result.singers), 2)
